@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: attack.c 25861 2016-07-28 14:33:01Z mattm $
+* $Id: attack.c 27443 2017-02-03 07:44:45Z kroosec $
 * Description: Launches the plugins, and manages multithreading.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -39,11 +39,10 @@
 #include <openvas/base/openvas_hosts.h>
 #include <openvas/base/openvas_networking.h>
 #include <openvas/misc/openvas_proctitle.h>
-#include <openvas/misc/kb.h>
+#include <openvas/base/kb.h>
 #include <openvas/misc/network.h>        /* for auth_printf */
 #include <openvas/misc/nvt_categories.h> /* for ACT_INIT */
 #include <openvas/misc/pcap_openvas.h>   /* for v6_is_local_ip */
-#include <openvas/misc/plugutils.h>      /* for plug_get_launch */
 #include <openvas/misc/prefs.h>          /* for prefs_get() */
 #include <openvas/misc/internal_com.h>
 
@@ -68,8 +67,6 @@
 
 #define MAX_FORK_RETRIES 10
 
-extern struct arglist *global_plugins;
-
 /**
  * Bundles information about target(s), configuration (globals arglist) and
  * scheduler.
@@ -174,82 +171,6 @@ network_scan_status (struct arglist *globals)
     return NSS_NONE;
 }
 
-/**
- * @brief Inits an arglist which can be used by the plugins.
- *
- * The arglist will have following keys and (type, value):
- *  - NAME (string, The hostname parameter)
- *  - FQDN (string, Fully qualified domain name, e.g. host.domain.net)
- *  - MAC  (string, The mac parameter if non-NULL)
- *  - IP   (*in_adrr, The ip parameter)
- *  - VHOSTS (string, comma separated list of vhosts for this IP)
- *
- * @param mac      MAC- adress of host or NULL.
- * @param hostname Hostname to be set.
- * @param ip       in_adress struct to be set.
- * @param vhosts   vhosts list to be set
- *
- * @return A 'hostinfo' arglist.
- */
-static struct arglist *
-attack_init_hostinfos_vhosts (char *mac, char *hostname, struct in6_addr *ip,
-                              char *vhosts, char *fqdn)
-{
-  struct arglist *hostinfos;
-
-  hostinfos = g_malloc0 (sizeof (struct arglist));
-  if (mac)
-    {
-      arg_add_value (hostinfos, "NAME", ARG_STRING, mac);
-      arg_add_value (hostinfos, "MAC", ARG_STRING, mac);
-    }
-  else
-    arg_add_value (hostinfos, "NAME", ARG_STRING, g_strdup (hostname));
-
-  if (fqdn)
-    arg_add_value (hostinfos, "FQDN", ARG_STRING, g_strdup (fqdn));
-  arg_add_value (hostinfos, "IP", ARG_PTR, ip);
-  if (vhosts)
-    arg_add_value (hostinfos, "VHOSTS", ARG_STRING, g_strdup (vhosts));
-  return (hostinfos);
-}
-
-/**
- * @brief Inits an arglist which can be used by the plugins.
- *
- * The arglist will have following keys and (type, value):
- *  - FQDN (string, Fully qualified domain name, e.g. host.domain.net)
- *  - NAME (string, The hostname parameter)
- *  - MAC  (string, The mac parameter if non-NULL)
- *  - IP   (*in_adrr, The ip parameter)
- *
- * @param mac      MAC- adress of host or NULL.
- * @param hostname Hostname to be set.
- * @param ip       in_adress struct to be set.
- *
- * @return A 'hostinfo' arglist.
- */
-static struct arglist *
-attack_init_hostinfos (char *mac, char *hostname, struct in6_addr *ip,
-                       char *fqdn)
-{
-  struct arglist *hostinfos;
-
-  hostinfos = g_malloc0 (sizeof (struct arglist));
-  if (mac)
-    {
-      arg_add_value (hostinfos, "NAME", ARG_STRING, mac);
-      arg_add_value (hostinfos, "MAC", ARG_STRING, mac);
-    }
-  else
-    arg_add_value (hostinfos, "NAME", ARG_STRING, g_strdup (hostname));
-  if (fqdn)
-    arg_add_value (hostinfos, "FQDN", ARG_STRING, g_strdup (fqdn));
-
-  arg_add_value (hostinfos, "IP", ARG_PTR, ip);
-  return (hostinfos);
-}
-
 int global_scan_stop = 0;
 
 static int
@@ -258,6 +179,24 @@ scan_is_stopped ()
  return global_scan_stop;
 }
 
+/**
+ * @brief Checks that an NVT category is safe.
+ *
+ * @param category  Category to check.
+ *
+ * @return 0 if category is unsafe, 1 otherwise.
+ */
+static int
+nvti_category_is_safe (int category)
+{
+  /* XXX: Duplicated from openvas-libraries. */
+  if (category == ACT_DESTRUCTIVE_ATTACK || category == ACT_KILL_HOST
+      || category == ACT_FLOOD || category == ACT_DENIAL)
+    return 0;
+  return 1;
+}
+
+
 /**
  * @brief Launches a nvt. Respects safe check preference (i.e. does not try
  * @brief destructive nvt if save_checks is yes).
@@ -271,49 +210,61 @@ scan_is_stopped ()
  */
 static int
 launch_plugin (struct arglist *globals, struct scheduler_plugin *plugin,
-               char *hostname, struct arglist *hostinfos, kb_t kb)
+               char *hostname, struct host_info *hostinfos, kb_t kb)
 {
-  struct arglist *args = plugin->arglist->value;
   int optimize = prefs_get_bool ("optimize_test");
-  int category = plugin->category;
+  int category;
+  char *oid, *name;
   gboolean network_scan = FALSE;
 
+  oid = plugin->oid;
+  category = nvticache_get_category (oid);
+  name = nvticache_get_name (oid);
   if (scan_is_stopped ())
     {
-      if (category != ACT_END)
+      if (category != ACT_LAST)
         {
           plugin->running_state = PLUGIN_STATUS_DONE;
+          g_free (name);
           return 0;
         }
       else
-        log_write ("Stopped scan wrap-up: Launching %s", plugin->arglist->name);
+        log_write ("Stopped scan wrap-up: Launching %s (%s)", name, oid);
     }
 
+  /* Stop the test if the host is 'dead' */
+  if (kb_item_get_int (kb, "Host/dead") > 0
+      || kb_item_get_int (kb, "Host/ping_failed") > 0)
+    {
+      log_write ("The remote host (%s) is dead", hostname);
+      pluginlaunch_stop (1);
+      plugin->running_state = PLUGIN_STATUS_DONE;
+      g_free (name);
+      return ERR_HOST_DEAD;
+    }
   if (network_scan_status (globals) == NSS_BUSY)
     network_scan = TRUE;
-  if (plug_get_launch (args) != LAUNCH_DISABLED)    /* can we launch it ? */
+
+  /* can we launch it ? */
+  if (plugin->enabled)
     {
       char *error;
 
-      if (prefs_get_bool ("safe_checks")
-          && (category == ACT_DESTRUCTIVE_ATTACK || category == ACT_KILL_HOST
-              || category == ACT_FLOOD || category == ACT_DENIAL))
+      if (prefs_get_bool ("safe_checks") && !nvti_category_is_safe (category))
         {
           if (prefs_get_bool ("log_whole_attack"))
             log_write
-              ("Not launching %s against %s %s (this is not an error)",
-               nvticache_get_filename ((const char *)plugin->arglist->name),
-               hostname,
-               "because safe checks are enabled");
+              ("Not launching %s (%s) against %s because safe checks are"
+               " enabled (this is not an error)", name, oid, hostname);
           plugin->running_state = PLUGIN_STATUS_DONE;
+          g_free (name);
           return 0;
         }
 
       if (network_scan)
         {
-          char asc_id[100], *oid;
+          char asc_id[100];
 
-          oid = plugin->arglist->name;
           assert (oid);
           snprintf (asc_id, sizeof (asc_id), "Launched/%s", oid);
 
@@ -322,9 +273,9 @@ launch_plugin (struct arglist *globals, struct scheduler_plugin *plugin,
               if (prefs_get_bool ("log_whole_attack"))
                 log_write ("Not launching %s against %s because it has already "
                            "been lanched in the past (this is not an error)",
-                           nvticache_get_filename ((const char *)plugin->arglist->name),
-                           hostname);
+                           oid, hostname);
               plugin->running_state = PLUGIN_STATUS_DONE;
+              g_free (name);
               return 0;
             }
           else
@@ -345,9 +296,8 @@ launch_plugin (struct arglist *globals, struct scheduler_plugin *plugin,
               || !(error = requirements_plugin (kb, plugin))))
         {
           int pid;
-          char *oid, *src;
+          char *src;
 
-          oid = plugin->arglist->name;
           src = nvticache_get_src (oid);
           /* Start the plugin */
           pid = plugin_launch (globals, plugin, hostinfos, kb, src);
@@ -355,38 +305,27 @@ launch_plugin (struct arglist *globals, struct scheduler_plugin *plugin,
           if (pid < 0)
             {
               plugin->running_state = PLUGIN_STATUS_UNRUN;
+              g_free (name);
               return ERR_CANT_FORK;
             }
 
           if (prefs_get_bool ("log_whole_attack"))
-            log_write ("Launching %s against %s [%d]",
-                       nvticache_get_filename ((const char *)plugin->arglist->name),
-                       hostname, pid);
-
-          /* Stop the test if the host is 'dead' */
-          if (kb_item_get_int (kb, "Host/dead") > 0
-              || kb_item_get_int (kb, "Host/ping_failed") > 0)
-            {
-              log_write ("The remote host (%s) is dead", hostname);
-              pluginlaunch_stop ();
-              plugin->running_state = PLUGIN_STATUS_DONE;
-              return ERR_HOST_DEAD;
-            }
+            log_write ("Launching %s (%s) against %s [%d]", name, oid, hostname,
+                       pid);
         }
       else                      /* requirements_plugin() failed */
         {
           plugin->running_state = PLUGIN_STATUS_DONE;
           if (prefs_get_bool ("log_whole_attack"))
             log_write
-              ("Not launching %s against %s %s (this is not an error)",
-               nvticache_get_filename ((const char *)plugin->arglist->name),
-               hostname,
-               error);
+              ("Not launching %s (%s) against %s %s (this is not an error)",
+               name, oid, hostname, error);
         }
     }                           /* if(plugins->launch) */
   else
     plugin->running_state = PLUGIN_STATUS_DONE;
 
+  g_free (name);
   return 0;
 }
 
@@ -425,7 +364,7 @@ kb_duplicate(kb_t dst, kb_t src, const gchar *filter)
  */
 static kb_t
 init_host_kb (struct arglist *globals, char *hostname,
-              struct arglist *hostinfos, kb_t *network_kb)
+              struct host_info *hostinfos, kb_t *network_kb)
 {
   kb_t kb;
   gchar *vhosts, *hostname_pattern, *hoststr;
@@ -467,10 +406,10 @@ init_host_kb (struct arglist *globals, char *hostname,
     }
 
   /* Add Hostname and Host-IP */
-  hoststr = arg_get_value (hostinfos, "FQDN");
+  hoststr = hostinfos->fqdn;
   if (hoststr)
     kb_item_add_str (kb, "Hostname", hoststr);
-  hostip = arg_get_value (hostinfos, "IP");
+  hostip = hostinfos->ip;
   if (hostip)
     {
       char ipstr[INET6_ADDRSTRLEN];
@@ -483,7 +422,7 @@ init_host_kb (struct arglist *globals, char *hostname,
     }
 
   /* If vhosts is set, split it and put it in the KB. */
-  vhosts = (gchar *)arg_get_value (hostinfos, "VHOSTS");
+  vhosts = hostinfos->vhosts;
   if (vhosts)
     {
       gchar **vhosts_array = g_strsplit (vhosts, ",", 0);
@@ -502,15 +441,14 @@ init_host_kb (struct arglist *globals, char *hostname,
  * @brief Attack one host.
  */
 static void
-attack_host (struct arglist *globals, struct arglist *hostinfos,
+attack_host (struct arglist *globals, struct host_info *hostinfos,
              char *hostname, plugins_scheduler_t sched, kb_t *net_kb)
 {
   /* Used for the status */
   int num_plugs, forks_retry = 0, global_socket;
   kb_t kb;
-  struct arglist *plugins = global_plugins;
 
-  proctitle_set ("openvassd: testing %s", arg_get_value (hostinfos, "NAME"));
+  proctitle_set ("openvassd: testing %s", hostinfos->name);
 
   global_socket = arg_get_value_int (globals, "global_socket");
   kb = init_host_kb (globals, hostname, hostinfos, net_kb);
@@ -519,11 +457,9 @@ attack_host (struct arglist *globals, struct arglist *hostinfos,
 
   kb_lnk_reset (kb);
 
-  num_plugs = get_active_plugins_number (plugins);
-
   /* launch the plugins */
-  pluginlaunch_init ();
-
+  pluginlaunch_init (hostinfos->name);
+  num_plugs = plugins_scheduler_count_active (sched);
   for (;;)
     {
       struct scheduler_plugin *plugin;
@@ -533,7 +469,7 @@ attack_host (struct arglist *globals, struct arglist *hostinfos,
       parent = getppid ();
       if (parent <= 1 || process_alive (parent) == 0)
         {
-          pluginlaunch_stop ();
+          pluginlaunch_stop (1);
           return;
         }
 
@@ -551,7 +487,19 @@ attack_host (struct arglist *globals, struct arglist *hostinfos,
                * Remote host died
                */
               if (e == ERR_HOST_DEAD)
-                goto host_died;
+                {
+                  char buffer[2048];
+                  snprintf
+                   (buffer, sizeof (buffer),
+                    "SERVER <|> LOG <|> %s <|> general/Host_Details"
+                    " <|> <host><detail><name>Host dead</name>"
+                    "<value>1</value><source><description/><type/>"
+                    "<name/></source></detail></host> <|>  <|> SERVER\n",
+                    hostname ?: "");
+
+                  internal_send (global_socket, buffer, INTERNAL_COMM_MSG_TYPE_DATA);
+                  goto host_died;
+                }
               else if (e == ERR_CANT_FORK)
                 {
                   if (forks_retry < MAX_FORK_RETRIES)
@@ -577,7 +525,7 @@ attack_host (struct arglist *globals, struct arglist *hostinfos,
               if (comm_send_status
                    (global_socket, hostname, cur_plug, num_plugs) < 0)
                 {
-                  pluginlaunch_stop ();
+                  pluginlaunch_stop (1);
                   goto host_died;
                 }
             }
@@ -594,7 +542,7 @@ attack_host (struct arglist *globals, struct arglist *hostinfos,
     comm_send_status (global_socket, hostname, num_plugs, num_plugs);
 
 host_died:
-  pluginlaunch_stop ();
+  pluginlaunch_stop (1);
   plugins_scheduler_free (sched);
 
   if (net_kb == NULL || kb != *net_kb)
@@ -608,11 +556,9 @@ static void
 attack_start (struct attack_start_args *args)
 {
   struct arglist *globals = args->globals;
-  char host_str[INET6_ADDRSTRLEN];
-  char *mac = args->host_mac_addr;
-  struct arglist *plugs = global_plugins;
+  char *host_str;
   struct in6_addr *hostip = &args->hostip;
-  struct arglist *hostinfos;
+  struct host_info *hostinfos;
   const char *non_simult = prefs_get ("non_simult_ports");
   const char *vhosts = prefs_get ("vhosts");
   const char *vhosts_ip = prefs_get ("vhosts_ip");
@@ -621,13 +567,13 @@ attack_start (struct attack_start_args *args)
   plugins_scheduler_t sched = args->sched;
   kb_t *net_kb = args->net_kb;
 
+  nvticache_reset ();
   /* Stringify the IP address. */
-  if (IN6_IS_ADDR_V4MAPPED (&args->hostip))
-    inet_ntop (AF_INET, ((char *)(&args->hostip))+12, host_str,
-               sizeof (host_str));
+  if (args->host_mac_addr)
+    host_str = g_strdup (args->host_mac_addr);
   else
-    inet_ntop (AF_INET6, &args->hostip, host_str, sizeof (host_str));
-
+    host_str = addr6_as_str (&args->hostip);
+  g_free (args->host_mac_addr);
   close (args->parent_socket);
   thread_socket = args->thread_socket;
   gettimeofday (&then, NULL);
@@ -642,35 +588,23 @@ attack_start (struct attack_start_args *args)
   arg_set_value (globals, "global_socket", GSIZE_TO_POINTER (thread_socket));
 
   if (vhosts == NULL || vhosts_ip == NULL)
-    hostinfos = attack_init_hostinfos (mac, host_str, hostip, args->fqdn);
+    hostinfos = host_info_init (host_str, hostip, NULL, args->fqdn);
   else
     {
       char *txt_ip;
-      struct in_addr inaddr;
-      inaddr.s_addr = hostip->s6_addr32[3];
 
-      if (IN6_IS_ADDR_V4MAPPED (hostip))
-        txt_ip = g_strdup (inet_ntoa (inaddr));
-      else
-        {
-          char name[INET6_ADDRSTRLEN];
-          txt_ip = g_strdup (inet_ntop (AF_INET6, hostip, name, sizeof (name)));
-        }
+      txt_ip = addr6_as_str (hostip);
       if (strcmp (vhosts_ip, txt_ip) != 0)
         vhosts = NULL;
       g_free (txt_ip);
-      hostinfos = attack_init_hostinfos_vhosts (mac, host_str, hostip, (char *)vhosts,
-                                                args->fqdn);
+      hostinfos = host_info_init (host_str, hostip, vhosts, args->fqdn);
     }
 
-  if (mac)
-    strcpy (host_str, mac);
-
-  plugins_set_socket (plugs, thread_socket);
   ntp_timestamp_host_scan_starts (thread_socket, host_str);
 
   // Start scan
   attack_host (globals, hostinfos, host_str, sched, net_kb);
+  host_info_free (hostinfos);
 
   if (!scan_is_stopped ())
     {
@@ -690,6 +624,7 @@ attack_start (struct attack_start_args *args)
   shutdown (thread_socket, 2);
   close (thread_socket);
   g_free (args->fqdn);
+  g_free (host_str);
 }
 
 static void
@@ -911,7 +846,7 @@ check_kb_access (int soc)
 static void
 handle_scan_stop_signal ()
 {
-  pluginlaunch_stop ();
+  pluginlaunch_stop (0);
   global_scan_stop = 1;
 }
 
@@ -928,7 +863,6 @@ attack_network (struct arglist *globals, kb_t *network_kb)
   openvas_hosts_t *sys_hosts_allow, *sys_hosts_deny;
   openvas_host_t *host;
   int global_socket = -1;
-  struct arglist *plugins = NULL;
   plugins_scheduler_t sched;
   int fork_retries = 0;
   GHashTable *files;
@@ -979,8 +913,6 @@ attack_network (struct arglist *globals, kb_t *network_kb)
 
   global_socket = arg_get_value_int (globals, "global_socket");
 
-  plugins = global_plugins;
-
   if (check_kb_access(global_socket))
       return;
 
@@ -1003,8 +935,9 @@ attack_network (struct arglist *globals, kb_t *network_kb)
     }
 
   /* Initialize the attack. */
-  sched = plugins_scheduler_init (plugins,
-    prefs_get_bool ("auto_enable_dependencies"), network_phase);
+  sched = plugins_scheduler_init
+           (prefs_get ("plugin_set"), prefs_get_bool ("auto_enable_dependencies"),
+            network_phase);
 
   max_hosts = get_max_hosts_number ();
   max_checks = get_max_checks_number ();
@@ -1068,7 +1001,6 @@ attack_network (struct arglist *globals, kb_t *network_kb)
   openvas_signal (SIGUSR1, handle_scan_stop_signal);
   while (host && !scan_is_stopped ())
     {
-      int pid;
       char *hostname;
       struct in6_addr host_ip;
 
@@ -1103,15 +1035,14 @@ attack_network (struct arglist *globals, kb_t *network_kb)
         }
       else
         {
+          int pid;
           struct attack_start_args args;
           char *MAC = NULL, *txt_ip;
-          int mac_err = -1;
           int soc[2];
 
           if (prefs_get_bool ("use_mac_addr") && v6_is_local_ip (&host_ip))
             {
-              mac_err = v6_get_mac_addr (&host_ip, &MAC);
-              if (mac_err > 0)
+              if (v6_get_mac_addr (&host_ip, &MAC) > 0)
                 {
                   /* remote host is down */
                   g_free (hostname);
