@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: comm.h 21962 2015-03-27 08:53:21Z kroosec $
+* $Id: comm.h 22633 2015-06-11 20:28:56Z kroosec $
 * Description: comm.c header.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -37,8 +37,9 @@ int comm_loading (int);
 void comm_terminate (int);
 void comm_send_preferences (int);
 int comm_wait_order (struct arglist *);
-void comm_setup_plugins (char *);
 void comm_send_nvt_info (int);
-void plugin_send_infos (int, char *);
+
+void
+send_plug_info (int, const char *);
 
 #endif
