@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: hosts.c 22719 2015-07-08 09:56:25Z mwiegand $
+* $Id: hosts.c 26458 2016-10-24 00:00:48Z kroosec $
 * Description: Basically creates a new process for each tested host.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -62,8 +62,8 @@ static int g_max_hosts = 15;
 static int
 forward (int in, int out)
 {
-  static char *buf = NULL;
-  static int bufsz = 0;
+  char *buf = NULL;
+  int bufsz = 0;
   int len;
   int type;
 
@@ -92,7 +92,7 @@ forward (int in, int out)
           e = nsend (out, buf + n, len - n, 0);
           if (e < 0 && errno == EINTR)
             continue;
-          else if (e <= 0)
+          else if (e < 0)
             return -1;
           else
             n += e;
@@ -100,7 +100,6 @@ forward (int in, int out)
     }
 
   g_free (buf);
-  buf = NULL;
   return 0;
 }
 
@@ -313,33 +312,28 @@ static int
 hosts_read_client (struct arglist *globals)
 {
   struct timeval tv;
-  int e;
+  int e = 0;
   fd_set rd;
-  int rsoc;
 
   if (g_soc == -1)
     return 0;
 
 
-  rsoc = openvas_get_socket_from_connection (g_soc);
-  if (rsoc == -1)
-    return -1;
-
   FD_ZERO (&rd);
-  FD_SET (rsoc, &rd);
+  FD_SET (g_soc, &rd);
 
   for (;;)
     {
       tv.tv_sec = 0;
       tv.tv_usec = 1000;
-      e = select (rsoc + 1, &rd, NULL, NULL, &tv);
+      e = select (g_soc + 1, &rd, NULL, NULL, &tv);
       if (e < 0 && errno == EINTR)
         continue;
       else
         break;
     }
 
-  if (e > 0 && FD_ISSET (rsoc, &rd) != 0)
+  if (e > 0 && FD_ISSET (g_soc, &rd) != 0)
     {
       int result;
       char buf[4096];
