@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: hosts.h 7203 2010-04-01 13:38:18Z mattm $
+* $Id: hosts.h 21716 2015-02-25 13:55:10Z kroosec $
 * Description: hosts.c header.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -32,12 +32,9 @@
 #define HOSTS_H
 
 int hosts_init (int, int);
-int hosts_new (struct arglist *, char *);
+int hosts_new (struct arglist *, char *, int);
 int hosts_set_pid (char *, pid_t);
 int hosts_read (struct arglist *);
-void hosts_pause_all ();
-void hosts_resume_all ();
-void hosts_stop_all ();
-int hosts_stop_host (struct arglist *globals, char *);
+void hosts_stop_all (void);
 
 #endif
