@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: nasl_plugins.c 26177 2016-09-01 16:07:30Z timopollmeier $
+* $Id: nasl_plugins.c 25468 2016-06-08 19:50:28Z hdoreau $
 * Description: Launches NASL plugins.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -50,13 +50,42 @@
 #include <openvas/misc/prefs.h>         /* for prefs_get_bool */
 
 #include "pluginload.h"
-#include "pluginscheduler.h"    /* for LAUNCH_DISABLED */
+#include "pluginscheduler.h"
 #include "pluginlaunch.h"
 #include "processes.h"
 #include "log.h"
 
 /**
- * @brief Add *one* .nasl plugin to the plugin list and return the pointer to it.
+ * @brief Add a nvti's preferences to the global preferences.
+ *
+ * @param nvti  NVTI pointer.
+ */
+static void
+prefs_add_nvti (const nvti_t *nvti)
+{
+  unsigned int i;
+
+  if (!nvti)
+    return;
+
+  for (i = 0; i < nvti_pref_len (nvti); i++)
+    {
+      char *pref, *cname;
+      const nvtpref_t *np = nvti_pref (nvti, i);
+
+      cname = g_strdup (nvtpref_name (np));
+      g_strchomp (cname);
+      pref = g_strdup_printf ("%s[%s]:%s", nvti_name (nvti), nvtpref_type (np),
+                              cname);
+      prefs_set (pref, nvtpref_default (np));
+
+      g_free (cname);
+      g_free (pref);
+    }
+}
+
+/**
+ * @brief Add *one* .nasl plugin to the plugin list.
  *
  * The plugin is first attempted to be loaded from the cache.
  * If that fails, it is parsed (via exec_nasl_script) and
@@ -66,17 +95,13 @@
  *
  * @param folder  Path to the plugin folder.
  * @param name    File-name of the plugin
- * @param plugins The arglist that the plugin shall be added to (with parameter
- *                name as the key).
  *
- * @return Pointer to the plugin (as arglist). NULL in case of errors.
+ * @return 0 on success, -1 on error.
  */
-struct arglist *
-nasl_plugin_add (char *folder, char *name, struct arglist *plugins)
+int
+nasl_plugin_add (char *folder, char *name)
 {
   char fullname[PATH_MAX + 1];
-  struct arglist *plugin_args;
-  struct arglist *prev_plugin = NULL;
   int nasl_mode;
   nasl_mode = NASL_EXEC_DESCR;
   nvti_t *nvti;
@@ -89,10 +114,10 @@ nasl_plugin_add (char *folder, char *name, struct arglist *plugins)
     }
 
   nvti = nvticache_get (name);
-  plugin_args = plug_create_from_nvti_and_prefs (nvti);
-  if (plugin_args == NULL)
+  if (nvti == NULL)
     {
       nvti_t *new_nvti;
+      struct arglist *plugin_args;
 
       g_free (nvti);
       plugin_args = g_malloc0 (sizeof (struct arglist));
@@ -103,13 +128,9 @@ nasl_plugin_add (char *folder, char *name, struct arglist *plugins)
         {
           log_write ("%s: Could not be loaded", fullname);
           arg_free_all (plugin_args);
-          return NULL;
+          return -1;
         }
-
-      // this extra pointer was only necessary during parsing the
-      // description part of a NASL. Now we can remove it from the args.
-      if (arg_get_value (plugin_args, "NVTI"))
-        arg_del_value (plugin_args, "NVTI");
+      arg_free_all (plugin_args);
 
       // Check mtime of plugin before caching it
       // Set to now if mtime is in the future
@@ -130,9 +151,7 @@ nasl_plugin_add (char *folder, char *name, struct arglist *plugins)
       if (nvti_oid (new_nvti) != NULL)
         {
           nvticache_add (new_nvti, name);
-          arg_free_all (plugin_args);
           nvti = nvticache_get (name);
-          plugin_args = plug_create_from_nvti_and_prefs (nvti);
         }
       else
         // Most likely an exit was hit before the description could be parsed.
@@ -141,44 +160,31 @@ nasl_plugin_add (char *folder, char *name, struct arglist *plugins)
       nvti_free (new_nvti);
     }
 
-  if (plugin_args == NULL)
+  if (nvti == NULL)
     {
       /* Discard invalid plugins */
       log_write ("%s: Failed to load", name);
-      g_free (nvti);
-      return NULL;
+      return -1;
     }
 
   if (nvti_oid (nvti) == NULL)
     {
       /* Discard invalid plugins */
       log_write ("%s: Failed to load, no OID", name);
-      plugin_free (plugin_args);
-      g_free (nvti);
-      return NULL;
-    }
-
-  plug_set_launch (plugin_args, LAUNCH_DISABLED);
-  prev_plugin = arg_get_value (plugins, nvti_oid (nvti));
-
-  // Was a plugin with the same oid already loaded? If so, remove it.
-  if (prev_plugin == NULL)
-    arg_add_value (plugins, nvti_oid (nvti), ARG_ARGLIST, plugin_args);
-  else
-    {
-      plugin_free (prev_plugin);
-      arg_set_value (plugins, nvti_oid (nvti), plugin_args);
+      nvti_free (nvti);
+      return -1;
     }
+  prefs_add_nvti (nvti);
 
   nvti_free (nvti);
-
-  return plugin_args;
+  return 0;
 }
 
 struct nasl_thread_args {
   struct arglist *args;
   char *name;
   const char *oid;
+  int soc;
 };
 
 static void
@@ -188,26 +194,25 @@ nasl_thread (struct nasl_thread_args *);
  * @brief Launch a NASL plugin.
  */
 int
-nasl_plugin_launch (struct arglist *globals, struct arglist *plugin,
-                    struct arglist *hostinfos, kb_t kb, char *name,
-                    const char *oid)
+nasl_plugin_launch (struct arglist *globals, struct host_info *hostinfo,
+                    kb_t kb, char *name, const char *oid, int soc)
 {
   int module;
   struct nasl_thread_args nargs;
+  struct arglist *plugin;
 
-  arg_add_value (plugin, "HOSTNAME", ARG_ARGLIST, hostinfos);
-  if (arg_get_value (plugin, "globals"))
-    arg_set_value (plugin, "globals", globals);
-  else
-    arg_add_value (plugin, "globals", ARG_ARGLIST, globals);
-
+  plugin = g_malloc0 (sizeof (struct arglist));
+  arg_add_value (plugin, "HOSTNAME", ARG_PTR, hostinfo);
+  arg_add_value (plugin, "globals", ARG_ARGLIST, globals);
   arg_add_value (plugin, "key", ARG_PTR, kb);
 
   nargs.args = plugin;
   nargs.name = name;
   nargs.oid = oid;
+  nargs.soc = soc;
 
   module = create_process ((process_func_t) nasl_thread, &nargs);
+  arg_free (plugin);
   return module;
 }
 
@@ -216,11 +221,13 @@ nasl_thread (struct nasl_thread_args *nargs)
 {
   struct arglist *args = nargs->args;
   struct arglist *globals = arg_get_value (args, "globals");
+  struct host_info *hostinfo = arg_get_value (args, "HOSTNAME");
   char *name = nargs->name;
-  int nasl_mode = 0, soc;
+  int nasl_mode = 0;
   kb_t kb;
   GError *error = NULL;
 
+  nvticache_reset ();
   if (prefs_get_bool ("be_nice"))
     {
       int nice_retval;
@@ -235,11 +242,8 @@ nasl_thread (struct nasl_thread_args *nargs)
   pluginlaunch_child_cleanup ();
   kb = arg_get_value (args, "key");
   kb_lnk_reset (kb);
-  soc = arg_get_value_int (args, "SOCKET");
-  arg_set_value (globals, "global_socket", GSIZE_TO_POINTER (soc));
-  proctitle_set ("openvassd: testing %s (%s)",
-                 arg_get_value (arg_get_value (args, "HOSTNAME"), "NAME"),
-                 name);
+  arg_set_value (globals, "global_socket", GSIZE_TO_POINTER (nargs->soc));
+  proctitle_set ("openvassd: testing %s (%s)", hostinfo->name, name);
 
   if (prefs_get_bool ("nasl_no_signature_check"))
     nasl_mode |= NASL_ALWAYS_SIGNED;
@@ -256,6 +260,6 @@ nasl_thread (struct nasl_thread_args *nargs)
     }
 
   exec_nasl_script (args, name, nargs->oid, nasl_mode);
-  internal_send (soc, NULL,
+  internal_send (nargs->soc, NULL,
                  INTERNAL_COMM_MSG_TYPE_CTRL | INTERNAL_COMM_CTRL_FINISHED);
 }
