@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: openvassd.c 25076 2016-04-25 12:49:37Z kroosec $
+* $Id: openvassd.c 26738 2016-11-29 14:47:17Z kroosec $
 * Description: Runs the OpenVAS-scanner.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -47,13 +47,17 @@
 #include <signal.h>    /* for SIGTERM */
 #include <netdb.h>     /* for addrinfo */
 #include <sys/wait.h>     /* for waitpid */
+#include <sys/un.h>
+#include <sys/stat.h>
+#include <pwd.h>
+#include <grp.h>
 
-#include <openvas/misc/network.h>    /* for ovas_scanner_context_t */
 #include <openvas/misc/openvas_proctitle.h> /* for proctitle_set */
 #include <openvas/misc/openvas_logging.h>  /* for setup_legacy_log_handler */
 #include <openvas/base/pidfile.h>    /* for pidfile_create */
 #include <openvas/base/nvticache.h> /* nvticache_free */
-#include <openvas/misc/kb.h>         /* for KB_PATH_DEFAULT */
+#include <openvas/base/kb.h>         /* for KB_PATH_DEFAULT */
+#include <openvas/base/gpgme_util.h>
 #include <openvas/misc/prefs.h>      /* for prefs_get() */
 
 #include <gcrypt.h> /* for gcry_control */
@@ -67,16 +71,17 @@
 #include "utils.h"        /* for wait_for_children1 */
 #include "pluginlaunch.h" /* for init_loading_shm */
 
+#ifdef SVN_REV_AVAILABLE
+#include "svnrevision.h"
+#endif
+
 /**
  * Globals that should not be touched (used in utils module).
  */
 int global_max_hosts = 15;
 int global_max_checks = 10;
 
-static int global_iana_socket;
-struct arglist *global_plugins;
-
-static GHashTable *global_options;
+static int global_iana_socket = -1;
 
 static volatile int loading_stop_signal = 0;
 static volatile int reload_signal = 0;
@@ -117,22 +122,15 @@ static openvassd_option openvassd_defaults[] = {
   {"unscanned_closed", "yes"},
   {"unscanned_closed_udp", "yes"},
   // Empty options must be "\0", not NULL, to match the behavior of
-  // prefs_init.
   {"vhosts", "\0"},
   {"vhosts_ip", "\0"},
   {"report_host_details", "yes"},
-  {"cert_file", SCANNERCERT},
-  {"key_file", SCANNERKEY},
-  {"ca_file", CACERT},
   {"kb_location", KB_PATH_DEFAULT},
   {"timeout_retry", "3"},
   {NULL, NULL}
 };
 
-/**
- * SSL context may be kept once it is inited.
- */
-static ovas_scanner_context_t ovas_scanner_ctx;
+gchar *unix_socket_path = NULL;
 
 static void
 start_daemon_mode (void)
@@ -235,18 +233,14 @@ handle_termination_signal (int sig)
 static void
 loading_client_handle (int soc)
 {
-  int soc2, opt = 1;
+  int opt = 1;
   if (soc <= 0)
     return;
-  soc2 = ovas_scanner_context_attach (ovas_scanner_ctx, soc);
-  if (soc2 < 0)
-    {
-      close (soc);
-      return;
-    }
+
   setsockopt (soc, SOL_SOCKET, SO_KEEPALIVE, &opt, sizeof (opt));
-  comm_loading (soc2);
-  close_stream_connection (soc2);
+  comm_loading (soc);
+  shutdown (soc, 2);
+  close (soc);
 }
 
 /*
@@ -278,7 +272,6 @@ loading_handler_start ()
 
   init_loading_shm ();
   parent_pid = getpid ();
-  openvas_signal (SIGTERM, remove_pidfile);
   child_pid = fork ();
   if (child_pid != 0)
     return child_pid;
@@ -304,13 +297,13 @@ loading_handler_start ()
   while (1)
     {
       unsigned int lg_address;
-      struct sockaddr_in6 address6;
+      struct sockaddr_un address;
       int soc;
 
       if (loading_stop_signal || kill (parent_pid, 0) < 0)
         break;
-      lg_address = sizeof (struct sockaddr_in6);
-      soc = accept (global_iana_socket, (struct sockaddr *) (&address6),
+      lg_address = sizeof (struct sockaddr_un);
+      soc = accept (global_iana_socket, (struct sockaddr *) (&address),
                     &lg_address);
       loading_client_handle (soc);
       sleep (1);
@@ -329,10 +322,7 @@ loading_handler_start ()
 void
 loading_handler_stop (pid_t handler_pid)
 {
-  int status;
-
-  kill (handler_pid, SIGTERM);
-  waitpid (handler_pid, &status, 0);
+  terminate_process (handler_pid);
   destroy_loading_shm ();
 }
 
@@ -353,38 +343,37 @@ init_signal_handlers ()
 static void
 reload_openvassd ()
 {
-  struct arglist *plugins;
   const char *config_file;
   pid_t handler_pid;
-  int i;
+  int i, ret;
 
-  log_write ("Reloading the scanner.");
   /* Ignore SIGHUP while reloading. */
   openvas_signal (SIGHUP, SIG_IGN);
 
+  /* Reinitialize logging before writing to it. */
+  log_init (prefs_get ("logfile"));
+  log_write ("Reloading the scanner.");
+
   handler_pid = loading_handler_start ();
   if (handler_pid < 0)
     return;
   /* Reload config file. */
   config_file = prefs_get ("config_file");
-  prefs_init ();
   for (i = 0; openvassd_defaults[i].option != NULL; i++)
     prefs_set (openvassd_defaults[i].option, openvassd_defaults[i].value);
   prefs_config (config_file);
 
   /* Reload the plugins */
   nvticache_free ();
-  plugins = plugins_init ();
+  ret = plugins_init ();
   set_globals_from_preferences ();
-  plugins_free (global_plugins);
-  global_plugins = plugins;
   loading_handler_stop (handler_pid);
-  if (!global_plugins)
-    exit (1);
 
   log_write ("Finished reloading the scanner.");
   reload_signal = 0;
   openvas_signal (SIGHUP, handle_reload_signal);
+  if (ret)
+    exit (1);
 }
 
 static void
@@ -411,36 +400,17 @@ handle_client (struct arglist *globals)
 static void
 scanner_thread (struct arglist *globals)
 {
-  char asciiaddr[INET6_ADDRSTRLEN];
-  int opt = 1, soc2 = -1, nice_retval, soc;
-  struct sockaddr_storage addr;
-  socklen_t len;
+  int opt = 1, soc;
 
+  nvticache_reset ();
   soc = arg_get_value_int (globals, "global_socket");
-  len = sizeof (addr);
-  getpeername (soc, (struct sockaddr *) &addr, &len);
-  if (addr.ss_family == AF_INET)
-    {
-      struct sockaddr_in *saddr = (struct sockaddr_in *) &addr;
-      inet_ntop (AF_INET,  &saddr->sin_addr, asciiaddr, sizeof(asciiaddr));
-    }
-  else
-    {
-      struct sockaddr_in6 *s6addr = (struct sockaddr_in6 *) &addr;
-      if (IN6_IS_ADDR_V4MAPPED (&s6addr->sin6_addr))
-        inet_ntop (AF_INET, &s6addr->sin6_addr.s6_addr[12],
-                   asciiaddr, sizeof (asciiaddr));
-      else
-        inet_ntop (AF_INET6, &s6addr->sin6_addr, asciiaddr, sizeof (asciiaddr));
-    }
-  proctitle_set ("openvassd: Serving %s", asciiaddr);
+  proctitle_set ("openvassd: Serving %s", unix_socket_path);
 
   /* Everyone runs with a nicelevel of 10 */
   if (prefs_get_bool ("be_nice"))
     {
       errno = 0;
-      nice_retval = nice (10);
-      if (nice_retval == -1 && errno != 0)
+      if (nice(10) == -1 && errno != 0)
         {
           log_write ("Unable to renice process: %d", errno);
         }
@@ -449,85 +419,27 @@ scanner_thread (struct arglist *globals)
   /* Close the scanner thread - it is useless for us now */
   close (global_iana_socket);
 
-  soc2 = ovas_scanner_context_attach (ovas_scanner_ctx, soc);
-  if (soc2 < 0)
+  if (soc < 0)
     goto shutdown_and_exit;
 
-  /* FIXME: The pre-gnutls code optionally printed information about
-   * the peer's certificate at this point.
-   */
-
   setsockopt (soc, SOL_SOCKET, SO_KEEPALIVE, &opt, sizeof (opt));
   /* arg_set_value *replaces* an existing value, but it shouldn't fail here */
   arg_add_value (globals, "parent_socket", ARG_INT, GSIZE_TO_POINTER (soc));
-  arg_set_value (globals, "global_socket", GSIZE_TO_POINTER (soc2));
+  arg_set_value (globals, "global_socket", GSIZE_TO_POINTER (soc));
 
-  if (comm_init (soc2) < 0)
-    {
-      close_stream_connection (soc);
-      exit (0);
-    }
+  if (comm_init (soc) < 0)
+    exit (0);
   handle_client (globals);
 
 shutdown_and_exit:
-  if (soc2 >= 0)
-    close_stream_connection (soc2);
-  else
-    {
-      shutdown (soc, 2);
-      close (soc);
-    }
+  shutdown (soc, 2);
+  close (soc);
 
   /* Kill left overs */
   end_daemon_mode ();
   exit (0);
 }
 
-static void
-init_ssl_ctx (const char *priority, const char *dhparams)
-{
-  if (openvas_SSL_init () < 0)
-    {
-      log_write ("Could not initialize openvas SSL!\n");
-      exit (1);
-    }
-
-  /* Only initialize ovas_scanner_ctx once */
-  if (ovas_scanner_ctx == NULL)
-    {
-      const char *cert, *key, *passwd, *ca_file;
-
-      ca_file = prefs_get ("ca_file");
-      if (ca_file == NULL)
-        {
-          log_write ("Missing ca_file - Did you run openvas-mkcert?\n");
-          exit (1);
-        }
-      cert = prefs_get ("cert_file");
-      if (cert == NULL)
-        {
-          log_write ("Missing cert_file - Did you run openvas-mkcert?\n");
-          exit (1);
-        }
-      key = prefs_get ("key_file");
-      if (key == NULL)
-        {
-          log_write ("Missing key_file - Did you run openvas-mkcert?\n");
-          exit (1);
-        }
-
-      passwd = prefs_get ("pem_password");
-      ovas_scanner_ctx = ovas_scanner_context_new
-                          (OPENVAS_ENCAPS_TLScustom, cert, key, passwd, ca_file,
-                           priority, dhparams);
-      if (!ovas_scanner_ctx)
-        {
-          log_write ("Could not create ovas_scanner_ctx");
-          exit (1);
-        }
-    }
-}
-
 /*
  * @brief Terminates the scanner if a termination signal was received.
  */
@@ -561,27 +473,30 @@ check_reload ()
 static void
 main_loop ()
 {
+#ifdef OPENVASSD_SVN_REVISION
+  log_write ("openvassd %s (SVN revision %i) started",
+             OPENVASSD_VERSION,
+             OPENVASSD_SVN_REVISION);
+#else
   log_write ("openvassd %s started", OPENVASSD_VERSION);
+#endif
   proctitle_set ("openvassd: Waiting for incoming connections");
   for (;;)
     {
       int soc;
       unsigned int lg_address;
-      struct sockaddr_in6 address6;
+      struct sockaddr_un address;
       struct arglist *globals;
 
       check_termination ();
       check_reload ();
       wait_for_children1 ();
-      lg_address = sizeof (struct sockaddr_in6);
-      soc = accept (global_iana_socket, (struct sockaddr *) (&address6),
+      lg_address = sizeof (struct sockaddr_un);
+      soc = accept (global_iana_socket, (struct sockaddr *) (&address),
                     &lg_address);
       if (soc == -1)
         continue;
 
-      /*
-       * MA: you cannot share an open SSL connection through fork/multithread
-       * The SSL connection shall be open _after_ the fork */
       globals = g_malloc0 (sizeof (struct arglist));
       arg_add_value (globals, "global_socket", ARG_INT, GSIZE_TO_POINTER (soc));
 
@@ -597,59 +512,97 @@ main_loop ()
 }
 
 /**
- * Initialization of the network :
- * we setup the socket that will listen for incoming connections on port \<port\>
- * on address \<addr\>
+ * Initialization of the network in unix socket case:
+ * we setup the socket that will listen for incoming connections on
+ * unix_socket_path.
  *
- * @param port Port on which to listen.
  * @param[out] sock Socket to be initialized.
- * @param addr Adress.
+ * @param unix_socket_path Path to unix socket to listen on.
  *
- * @return 0 on success. Exit(1)s on failure.
+ * @return 0 on success. -1 on failure.
  */
 static int
-init_network (int port, int *sock, struct addrinfo addr)
+init_unix_network (int *sock, const char *owner, const char *group,
+                   const char *mode)
 {
-  int option = 1;
+  struct sockaddr_un addr;
+  struct stat ustat;
+  int unix_socket;
+  mode_t omode;
 
-  if (addr.ai_family == AF_INET)
-    ((struct sockaddr_in *) (addr.ai_addr))->sin_port = htons (port);
-  else
-    ((struct sockaddr_in6 *) (addr.ai_addr))->sin6_port = htons (port);
-  if ((*sock = socket (addr.ai_family, SOCK_STREAM, 0)) == -1)
+  unix_socket = socket (AF_UNIX, SOCK_STREAM, 0);
+  if (unix_socket == -1)
     {
-      int ec = errno;
-      log_write ("socket(AF_INET): %s", strerror (ec));
-      exit (1);
+      log_write ("%s: Couldn't create UNIX socket", __FUNCTION__);
+      return -1;
+    }
+  addr.sun_family = AF_UNIX;
+  strncpy (addr.sun_path, unix_socket_path, sizeof (addr.sun_path));
+  if (!stat (addr.sun_path, &ustat))
+    {
+      /* Remove socket so we can bind(). */
+      unlink (addr.sun_path);
+    }
+  if (bind (unix_socket, (struct sockaddr *) &addr, sizeof (struct sockaddr_un))
+      == -1)
+    {
+      log_write ("%s: Error on bind(%s): %s", __FUNCTION__,
+                 unix_socket_path, strerror (errno));
+      return -1;
     }
 
-  setsockopt (*sock, SOL_SOCKET, SO_REUSEADDR, &option, sizeof (int));
-  if (bind (*sock, (struct sockaddr *) (addr.ai_addr), addr.ai_addrlen) == -1)
+  if (owner)
     {
-      log_write ("bind() failed : %s\n", strerror (errno));
-      exit (1);
+      struct passwd *pwd = getpwnam (owner);
+      if (!pwd)
+        {
+          log_write ("%s: User %s not found.", __FUNCTION__, owner);
+          return -1;
+        }
+      if (chown (unix_socket_path, pwd->pw_uid, -1) == -1)
+        {
+          log_write ("%s: chown: %s", __FUNCTION__, strerror (errno));
+          return -1;
+        }
     }
 
-  if (listen (*sock, 10) == -1)
+  if (group)
     {
-      log_write ("listen() failed : %s\n", strerror (errno));
-      shutdown (*sock, 2);
-      close (*sock);
-      exit (1);
+      struct group *grp = getgrnam (group);
+      if (!grp)
+        {
+          log_write ("%s: Group %s not found.", __FUNCTION__, group);
+          return -1;
+        }
+      if (chown (unix_socket_path, -1, grp->gr_gid) == -1)
+        {
+          log_write ("%s: chown: %s", __FUNCTION__, strerror (errno));
+          return -1;
+        }
     }
-  return (0);
-}
 
-static void
-init_plugins (GHashTable *options)
-{
-  struct arglist *plugins;
+  if (!mode)
+    mode = "660";
+ omode = strtol (mode, 0, 8);
+ if (omode <= 0 || omode > 4095)
+   {
+     log_write ("%s: Erroneous liste-mode value", __FUNCTION__);
+     return -1;
+   }
+ if (chmod (unix_socket_path, strtol (mode, 0, 8)) == -1)
+   {
+     log_write ("%s: chmod: %s", __FUNCTION__, strerror (errno));
+     return -1;
+   }
 
-  plugins = plugins_init ();
+  if (listen (unix_socket, 128) == -1)
+    {
+      log_write ("%s: Error on listen(): %s", __FUNCTION__, strerror (errno));
+      return -1;
+    }
 
-  g_hash_table_replace (options, "plugins", plugins);
-  plugins_free (global_plugins);
-  global_plugins = plugins;
+  *sock = unix_socket;
+  return 0;
 }
 
 /**
@@ -658,21 +611,10 @@ init_plugins (GHashTable *options)
  * @param stop_early 0: do some initialization, 1: no initialization.
  */
 static int
-init_openvassd (GHashTable *options, int first_pass, int stop_early,
-                int dont_fork)
+init_openvassd (int dont_fork, const char *config_file)
 {
-  int isck = -1;
-  int scanner_port;
-  char *config_file;
-  struct addrinfo *addr;
   int i;
 
-  scanner_port = GPOINTER_TO_SIZE (g_hash_table_lookup (options,
-                                                        "scanner_port"));
-  config_file = g_hash_table_lookup (options, "config_file");
-  addr = g_hash_table_lookup (options, "addr");
-
-  prefs_init ();
   for (i = 0; openvassd_defaults[i].option != NULL; i++)
     prefs_set (openvassd_defaults[i].option, openvassd_defaults[i].value);
   prefs_config (config_file);
@@ -681,14 +623,6 @@ init_openvassd (GHashTable *options, int first_pass, int stop_early,
   if (dont_fork == FALSE)
     setup_legacy_log_handler (log_vwrite);
 
-  if (!stop_early)
-    {
-      if (first_pass != 0)
-        init_network (scanner_port, &isck, *addr);
-    }
-
-  g_hash_table_replace (options, "isck", GSIZE_TO_POINTER (isck));
-
   set_globals_from_preferences ();
 
   return 0;
@@ -749,31 +683,19 @@ gcrypt_init ()
 int
 main (int argc, char *argv[])
 {
-  int exit_early = 0, scanner_port = 9391;
+  int ret;
   pid_t handler_pid;
-  char *myself;
-  GHashTable *options;
-  struct addrinfo *mysaddr;
-  struct addrinfo hints;
-  struct addrinfo ai;
-  struct sockaddr_in saddr;
-  struct sockaddr_in6 s6addr;
 
   proctitle_init (argc, argv);
   gcrypt_init ();
 
-  if ((myself = strrchr (*argv, '/')) == 0)
-    myself = *argv;
-  else
-    myself++;
-
   static gboolean display_version = FALSE;
   static gboolean dont_fork = FALSE;
-  static gchar *address = NULL;
-  static gchar *port = NULL;
   static gchar *config_file = NULL;
-  static gchar *gnutls_priorities = "NORMAL";
-  static gchar *dh_params = NULL;
+  static gchar *listen_owner = NULL;
+  static gchar *listen_group = NULL;
+  static gchar *listen_mode = NULL;
+  static gchar *gnupg_dir = NULL;
   static gboolean print_specs = FALSE;
   static gboolean print_sysconfdir = FALSE;
   static gboolean only_cache = FALSE;
@@ -784,10 +706,6 @@ main (int argc, char *argv[])
      "Display version information", NULL},
     {"foreground", 'f', 0, G_OPTION_ARG_NONE, &dont_fork,
      "Do not run in daemon mode but stay in foreground", NULL},
-    {"listen", 'a', 0, G_OPTION_ARG_STRING, &address,
-     "Listen on <address>", "<address>"},
-    {"port", 'p', 0, G_OPTION_ARG_STRING, &port,
-     "Use port number <number>", "<number>"},
     {"config-file", 'c', 0, G_OPTION_ARG_FILENAME, &config_file,
      "Configuration file", "<filename>"},
     {"cfg-specs", 's', 0, G_OPTION_ARG_NONE, &print_specs,
@@ -796,11 +714,17 @@ main (int argc, char *argv[])
      "Print system configuration directory (set at compile time)", NULL},
     {"only-cache", 'C', 0, G_OPTION_ARG_NONE, &only_cache,
      "Exit once the NVT cache has been initialized or updated", NULL},
-    {"gnutls-priorities", '\0', 0, G_OPTION_ARG_STRING, &gnutls_priorities,
-     "GnuTLS priorities string", "<string>"},
-    {"dh-params", '\0', 0, G_OPTION_ARG_STRING, &dh_params,
-     "Diffie-Hellman parameters file", "<string>"},
-    {NULL}
+    {"unix-socket", 'c', 0, G_OPTION_ARG_FILENAME, &unix_socket_path,
+     "Path of unix socket to listen on", "<filename>"},
+    {"listen-owner", '\0', 0, G_OPTION_ARG_STRING, &listen_owner,
+     "Owner of the unix socket", "<string>"},
+    {"listen-group", '\0', 0, G_OPTION_ARG_STRING, &listen_group,
+     "Group of the unix socket", "<string>"},
+    {"listen-mode", '\0', 0, G_OPTION_ARG_STRING, &listen_mode,
+     "File mode of the unix socket", "<string>"},
+    {"gnupg-home", 'c', 0, G_OPTION_ARG_STRING, &gnupg_dir,
+     "Gnupg home directory", "<directory>"},
+    {NULL, 0, 0, 0, NULL, NULL, NULL}
   };
 
   option_context =
@@ -827,65 +751,17 @@ main (int argc, char *argv[])
     }
   tzset ();
 
-  if (print_specs)
-    exit_early = 2;           /* no cipher initialization */
-
-  if (address != NULL)
-    {
-      memset (&hints, 0, sizeof (hints));
-      hints.ai_socktype = SOCK_STREAM;
-      hints.ai_protocol = IPPROTO_TCP;
-      hints.ai_flags = AI_NUMERICHOST;
-      if (getaddrinfo (address, NULL, &hints, &mysaddr))
-        {
-          printf ("Invalid IP address.\n");
-          printf ("Please use %s --help for more information.\n", myself);
-          exit (0);
-        }
-      /* deep copy */
-      ai.ai_family = mysaddr->ai_family;
-      if (ai.ai_family == AF_INET)
-        {
-          memcpy (&saddr, mysaddr->ai_addr, mysaddr->ai_addrlen);
-          ai.ai_addr = (struct sockaddr *) &saddr;
-        }
-      else
-        {
-          memcpy (&s6addr, mysaddr->ai_addr, mysaddr->ai_addrlen);
-          ai.ai_addr = (struct sockaddr *) &s6addr;
-        }
-      ai.ai_family = mysaddr->ai_family;
-      ai.ai_protocol = mysaddr->ai_protocol;
-      ai.ai_socktype = mysaddr->ai_socktype;
-      ai.ai_addrlen = mysaddr->ai_addrlen;
-      freeaddrinfo (mysaddr);
-    }
-  else
-    {
-      /* Default to IPv4 */
-      /*Warning: Not filling all the fields */
-      saddr.sin_addr.s_addr = INADDR_ANY;
-      saddr.sin_family = ai.ai_family = AF_INET;
-      ai.ai_addrlen = sizeof (saddr);
-      ai.ai_addr = (struct sockaddr *) &saddr;
-    }
-
-  if (port != NULL)
-    {
-      scanner_port = atoi (port);
-      if ((scanner_port <= 0) || (scanner_port >= 65536))
-        {
-          printf ("Invalid port specification.\n");
-          printf ("Please use %s --help for more information.\n", myself);
-          exit (1);
-        }
-    }
+  if (!unix_socket_path)
+    unix_socket_path = g_build_filename (OPENVAS_RUN_DIR, "openvassd.sock", NULL);
 
   if (display_version)
     {
       printf ("OpenVAS Scanner %s\n", OPENVASSD_VERSION);
+#ifdef OPENVASSD_SVN_REVISION
+      printf ("SVN revision %i\n", OPENVASSD_SVN_REVISION);
+#endif
       printf
-        ("Most new code since 2005: (C) 2015 Greenbone Networks GmbH\n");
+        ("Most new code since 2005: (C) 2016 Greenbone Networks GmbH\n");
       printf
         ("Nessus origin: (C) 2004 Renaud Deraison <deraison@nessus.org>\n");
       printf ("License GPLv2: GNU GPL version 2\n");
@@ -895,37 +771,37 @@ main (int argc, char *argv[])
       exit (0);
     }
 
-  options = g_hash_table_new (g_str_hash, g_str_equal);
+  if (gnupg_dir)
+    set_gpghome (gnupg_dir);
 
-  if (config_file != NULL)
-    g_hash_table_insert (options, "acc_hint", GSIZE_TO_POINTER(1));
-  else
+  if (!config_file)
     config_file = OPENVASSD_CONF;
-
-  g_hash_table_insert (options, "scanner_port",
-                       GSIZE_TO_POINTER (scanner_port));
-  g_hash_table_insert (options, "config_file", config_file);
-  g_hash_table_insert (options, "addr", &ai);
-
   if (only_cache)
     {
-      init_openvassd (options, 0, 1, dont_fork);
-      init_plugins (options);
-      exit (0);
+      if (init_openvassd (dont_fork, config_file))
+        return 1;
+      if (plugins_init ())
+        return 1;
+      return 0;
     }
 
-  init_openvassd (options, 1, exit_early, dont_fork);
-  global_options = options;
-  global_iana_socket = GPOINTER_TO_SIZE (g_hash_table_lookup (options, "isck"));
-  global_plugins = g_hash_table_lookup (options, "plugins");
+  if (init_openvassd (dont_fork, config_file))
+    return 1;
+  if (!print_specs)
+    {
+      if (init_unix_network (&global_iana_socket, listen_owner, listen_group,
+                             listen_mode))
+        return 1;
+    }
 
   /* special treatment */
   if (print_specs)
-    prefs_dump ();
-  if (exit_early)
-    exit (0);
+    {
+      prefs_dump ();
+      exit (0);
+    }
+  flush_all_kbs ();
 
-  init_ssl_ctx (gnutls_priorities, dh_params);
   // Daemon mode:
   if (dont_fork == FALSE)
     set_daemon_mode ();
@@ -933,13 +809,11 @@ main (int argc, char *argv[])
   handler_pid = loading_handler_start ();
   if (handler_pid < 0)
     return 1;
-  init_plugins (options);
+  ret = plugins_init ();
   loading_handler_stop (handler_pid);
-  if (!global_plugins)
+  if (ret)
     return 1;
-  flush_all_kbs ();
   init_signal_handlers ();
   main_loop ();
-  g_hash_table_destroy (options);
   exit (0);
 }
