@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginlaunch.h 20751 2014-11-02 18:58:52Z jan $
+* $Id: pluginlaunch.h 26467 2016-10-24 12:18:36Z kroosec $
 * Description: pluginlaunch.c header.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -33,12 +33,12 @@
 #include "pluginload.h"         /* for struct pl_class_t */
 #include "pluginscheduler.h"    /* for struct plugins_scheduler_t */
 
-void pluginlaunch_init (void);
+void pluginlaunch_init (const char *);
 void pluginlaunch_wait (void);
 void pluginlaunch_wait_for_free_process (void);
-void pluginlaunch_stop (void);
+void pluginlaunch_stop (int);
 int plugin_launch (struct arglist *, struct scheduler_plugin *,
-                   struct arglist *, kb_t, char *);
+                   struct host_info *, kb_t, char *);
 
 void pluginlaunch_child_cleanup (void);
 void pluginlaunch_disable_parrallel_checks (void);
