@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginload.c 22229 2015-04-28 09:58:15Z mwiegand $
+* $Id: pluginload.c 25551 2016-06-22 20:12:30Z jan $
 * Description: Loads plugins from disk into memory.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -217,8 +217,8 @@ set_total_loading_plugins (int total)
     loading_shm[1] = total;
 }
 
-static struct arglist *
-plugins_reload_from_dir (struct arglist *plugins, char *folder)
+static int
+plugins_reload_from_dir (char *folder)
 {
   GSList *files = NULL, *f;
   const gchar *pref_include_folders;
@@ -231,7 +231,7 @@ plugins_reload_from_dir (struct arglist *plugins, char *folder)
   if (pref_include_folders != NULL)
     {
       gchar **include_folders = g_strsplit (pref_include_folders, ":", 0);
-      int i = 0;
+      unsigned int i = 0;
 
       for (i = 0; i < g_strv_length (include_folders); i++)
         {
@@ -252,7 +252,7 @@ plugins_reload_from_dir (struct arglist *plugins, char *folder)
 #endif
       log_write ("Could not determine the value of <plugins_folder>. "
                  " Check %s\n", (char *) prefs_get ("config_file"));
-      return plugins;
+      return -1;
     }
 
   files = collect_nvts (folder, "", files);
@@ -290,7 +290,7 @@ plugins_reload_from_dir (struct arglist *plugins, char *folder)
         log_write ("Loading %s", name);
       if (g_str_has_suffix (name, ".nasl"))
         {
-          if (!nasl_plugin_add (folder, name, plugins))
+          if (nasl_plugin_add (folder, name))
             err_count++;
         }
 
@@ -298,109 +298,32 @@ plugins_reload_from_dir (struct arglist *plugins, char *folder)
         {
           log_write ("Stopped loading plugins: High number of errors.");
           proctitle_set ("openvassd: Error loading NVTs.");
-          plugins_free (plugins);
-#if GLIB_CHECK_VERSION (2, 28, 0)
           g_slist_free_full (files, g_free);
-#else
-          g_slist_foreach (files, (GFunc) g_free, NULL);
-          g_slist_free (files);
-#endif
-          return NULL;
+          return -1;
         }
       f = g_slist_next (f);
     }
 
-#if GLIB_CHECK_VERSION (2, 28, 0)
   g_slist_free_full (files, g_free);
-#else
-  g_slist_foreach (files, (GFunc) g_free, NULL);
-  g_slist_free (files);
-#endif
 
   proctitle_set ("openvassd: Reloaded all the NVTs.");
 
-  return plugins;
+  return 0;
 }
 
 /*
  * main function for loading all the plugins
  */
-struct arglist *
+int
 plugins_init (void)
 {
   const char *plugins_folder = prefs_get ("plugins_folder");
-  struct arglist *plugins = g_malloc0 (sizeof (struct arglist));
 
-  nvticache_init (prefs_get ("cache_folder"), plugins_folder);
-
-  return plugins_reload_from_dir (plugins, (char *)plugins_folder);
-}
-
-void
-plugin_set_socket (struct arglist *plugin, int soc)
-{
-  if (arg_get_value (plugin, "SOCKET") != NULL)
-    arg_set_value (plugin, "SOCKET", GSIZE_TO_POINTER (soc));
-  else
-    arg_add_value (plugin, "SOCKET", ARG_INT, GSIZE_TO_POINTER (soc));
-}
-
-int
-plugin_get_socket (struct arglist *plugin)
-{
-  return arg_get_value_int (plugin, "SOCKET");
-}
-
-
-void
-plugin_unlink (plugin)
-     struct arglist *plugin;
-{
-  if (plugin == NULL)
+  if (nvticache_init (prefs_get ("cache_folder"), plugins_folder,
+                      prefs_get ("kb_location")))
     {
-      log_write ("Error in plugin_unlink - args == NULL\n");
-      return;
-    }
-}
-
-
-void
-plugin_free (plugin)
-     struct arglist *plugin;
-{
-  plugin_unlink (plugin);
-  arg_free_all (plugin);
-}
-
-void
-plugins_free (plugins)
-     struct arglist *plugins;
-{
-  struct arglist *p = plugins;
-  if (p == NULL)
-    return;
-
-  while (p->next)
-    {
-      plugin_unlink (p->value);
-      p = p->next;
-    }
-  arg_free_all (plugins);
-}
-
-/*
- * Put our socket somewhere in the plugins
- * arguments
- */
-void
-plugins_set_socket (struct arglist *plugins, int soc)
-{
-  struct arglist *t;
-
-  t = plugins;
-  while (t && t->next)
-    {
-      plugin_set_socket (t->value, soc);
-      t = t->next;
+      log_write ("Failed to initialize nvti cache.");
+      return -1;
     }
+  return plugins_reload_from_dir ((char *)plugins_folder);
 }
