@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginload.h 20867 2014-11-16 20:23:56Z jan $
+* $Id: pluginload.h 24032 2016-01-04 21:56:25Z kroosec $
 * Description: pluginload.c header.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -32,14 +32,11 @@
 #define _OPENVAS_PLUGINLOAD_H
 
 #include <openvas/misc/arglists.h>   /* for struct arglist */
-#include <openvas/misc/kb.h>         /* for struct kb_item */
+#include <openvas/base/kb.h>         /* for struct kb_item */
+#include <openvas/misc/network.h>
 
-struct arglist *plugins_init (void);
-void plugin_set_socket (struct arglist *, int);
-int plugin_get_socket (struct arglist *);
-void plugins_set_socket (struct arglist *, int);
-void plugin_free (struct arglist *);
-void plugins_free (struct arglist *);
+int
+plugins_init (void);
 
 void
 init_loading_shm (void);
@@ -54,11 +51,11 @@ int
 total_loading_plugins (void);
 
 /* From nasl_plugins.c */
-struct arglist *
-nasl_plugin_add (char *, char *, struct arglist *);
+int
+nasl_plugin_add (char *, char *);
 
 int
-nasl_plugin_launch (struct arglist *, struct arglist *, struct arglist *,
-                    kb_t, char *, const char *);
+nasl_plugin_launch (struct arglist *, struct host_info *, kb_t, char *,
+                    const char *, int);
 
 #endif
