@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginscheduler.c 25491 2016-06-13 11:18:03Z kroosec $
+* $Id: pluginscheduler.c 26363 2016-10-11 13:16:43Z kroosec $
 * Description: Tells openvassd which plugin should be executed next.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -51,12 +51,8 @@
 
 struct hash
 {
-  char *name;
   struct scheduler_plugin *plugin;
-  gchar **dependencies;
-  int num_deps;
   struct hash **dependencies_ptr;
-  gchar **ports;
   struct hash *next;
 };
 
@@ -113,16 +109,7 @@ hash_link_destroy (struct hash *h)
     hash_link_destroy (h->next);
 
   g_free (h->dependencies_ptr);
-
-  g_strfreev (h->plugin->required_ports);
-  g_strfreev (h->plugin->required_udp_ports);
-  g_strfreev (h->plugin->required_keys);
-  g_strfreev (h->plugin->mandatory_keys);
-  g_strfreev (h->plugin->excluded_keys);
   g_free (h->plugin);
-
-  g_strfreev (h->dependencies);
-  g_strfreev (h->ports);
   g_free (h);
 }
 
@@ -140,34 +127,17 @@ hash_destroy (struct hash *h)
 
 
 static void
-hash_add (struct hash *h, char *name, struct scheduler_plugin *plugin)
+hash_add (struct hash *h, struct scheduler_plugin *plugin)
 {
   struct hash *l = g_malloc0 (sizeof (struct hash));
-  unsigned int idx = mkhash ((char *)nvticache_get_filename ((const char *)name));
-  nvti_t *nvti;
+  unsigned int idx = mkhash (plugin->oid);
 
-  nvti = nvticache_get_by_oid_full (plugin->arglist->name);
   l->plugin = plugin;
   l->plugin->parent_hash = l;
-  l->name = name;
   l->next = h[idx].next;
   h[idx].next = l;
   l->dependencies_ptr = NULL;
 
-  if (nvti_required_ports (nvti) != NULL)
-    l->ports = g_strsplit (nvti_required_ports (nvti), ", ", 0);
-  else
-    l->ports = NULL;
-
-  if (nvti_dependencies (nvti) != NULL)
-    {
-      l->dependencies = g_strsplit (nvti_dependencies (nvti), ", ", 0);
-      for (l->num_deps = 0; l->dependencies[l->num_deps] != NULL; l->num_deps ++) ;
-    }
-  else
-    l->dependencies = NULL;
-
-  nvti_free (nvti);
 }
 
 
@@ -178,7 +148,7 @@ _hash_get (struct hash *h, char *name)
   struct hash *l = h[idx].next;
   while (l != NULL)
     {
-      if (strcmp (nvticache_get_filename ((const char *)l->name), name) == 0)
+      if (strcmp (l->plugin->oid, name) == 0)
         return l;
       else
         l = l->next;
@@ -201,44 +171,47 @@ hash_get_deps_ptr (struct hash *h, char *name)
 static void
 hash_fill_deps (struct hash *h, struct hash *l)
 {
-  int i, j = 0;
+  int i, j = 0, num_deps;
+  char *dependencies, **array;
 
-  if (l->num_deps == 0)
+  if (!l->plugin)
+    return;
+  dependencies = nvticache_get_dependencies (l->plugin->oid);
+  if (!dependencies)
+    return;
+  array = g_strsplit (dependencies, ", ", 0);
+  g_free (dependencies);
+  if (!array)
     return;
 
-  l->dependencies_ptr =
-    g_malloc0 ((1 + l->num_deps) * sizeof (struct hash *));
-  for (i = 0; l->dependencies[i]; i++)
+  for (num_deps = 0; array[num_deps]; num_deps++)
+    ;
+  if (num_deps == 0)
+    return;
+
+  l->dependencies_ptr = g_malloc0 ((1 + num_deps) * sizeof (struct hash *));
+  for (i = 0; array[i]; i++)
     {
-      struct hash *d = _hash_get (h, l->dependencies[i]);
+      char *oid;
+      struct hash *d;
+
+      oid = nvticache_get_oid (array[i]);
+      if (!oid)
+        {
+          log_write ("scheduler: %s depends on %s which could not be found",
+                     l->plugin->oid, array[i]);
+          continue;
+        }
+      d = _hash_get (h, oid);
       if (d != NULL)
         l->dependencies_ptr[j++] = d;
       else
-        {
-          gchar *path = g_path_get_dirname (nvticache_get_filename ((const char *)l->plugin->arglist->name));
-          if (g_ascii_strcasecmp (path, ".") != 0)
-            {
-              gchar *dep_with_path =
-                g_build_filename (path, l->dependencies[i], NULL);
-              d = _hash_get (h, dep_with_path);
-              g_free (dep_with_path);
-            }
-          g_free (path);
-          if (d != NULL)
-            {
-              l->dependencies_ptr[j++] = d;
-            }
-          else
-            {
-              log_write ("scheduler: %s depends on %s which could not be found,"
-                         " thus this dependency is not considered for execution"
-                         " sequence",
-                         nvticache_get_filename ((const char *)l->plugin->arglist->name),
-                         l->dependencies[i]);
-            }
-        }
+        log_write ("scheduler: %s depends on %s which could not be found",
+                   l->plugin->oid, array[i]);
+      g_free (oid);
     }
   l->dependencies_ptr[j] = NULL;
+  g_strfreev (array);
 }
 
 /*----------------------------------------------------------------------*/
@@ -269,22 +242,27 @@ void
 scheduler_mark_running_ports (plugins_scheduler_t sched,
                               struct scheduler_plugin *plugin)
 {
-  char **ports = plugin->parent_hash->ports;
+  char *ports, **array;
   int i;
 
-  if (ports == NULL)
+  ports = nvticache_get_required_ports (plugin->oid);
+  if (!ports)
     return;
 
-  for (i = 0; ports[i] != NULL; i++)
+  array = g_strsplit (ports, ", ", 0);
+  g_free (ports);
+  if (!array)
+    return;
+  for (i = 0; array[i] != NULL; i++)
     {
-      struct plist *pl = pl_get (sched->plist, ports[i]);
+      struct plist *pl = pl_get (sched->plist, array[i]);
 
       if (pl != NULL)
         pl->occurences++;
       else
         {
           pl = g_malloc0 (sizeof (struct plist));
-          pl->name = g_strdup (ports[i]);
+          pl->name = g_strdup (array[i]);
           pl->occurences = 1;
           pl->next = sched->plist;
           if (sched->plist != NULL)
@@ -293,23 +271,27 @@ scheduler_mark_running_ports (plugins_scheduler_t sched,
           sched->plist = pl;
         }
     }
+  g_strfreev (array);
 }
 
 void
 scheduler_rm_running_ports (plugins_scheduler_t sched,
                             struct scheduler_plugin *plugin)
 {
-  char **ports;
+  char *ports, **array;
   int i;
 
-  ports = plugin->parent_hash->ports;
-
-  if (ports == NULL)
+  ports = nvticache_get_required_ports (plugin->oid);
+  if (!ports)
     return;
 
-  for (i = 0; ports[i] != NULL; i++)
+  array = g_strsplit (ports, ", ", 0);
+  g_free (ports);
+  if (!array)
+    return;
+  for (i = 0; array[i] != NULL; i++)
     {
-      struct plist *pl = pl_get (sched->plist, ports[i]);
+      struct plist *pl = pl_get (sched->plist, array[i]);
 
       if (pl != NULL)
         {
@@ -330,14 +312,15 @@ scheduler_rm_running_ports (plugins_scheduler_t sched,
         }
       else
         log_write ("Warning: scheduler_rm_running_ports failed ?! (%s)\n",
-                   ports[i]);
+                   array[i]);
     }
+  g_strfreev (array);
 }
 
 
 struct scheduler_plugin *
 plugin_next_unrun_dependency (plugins_scheduler_t sched,
-                               struct hash **dependencies_ptr)
+                              struct hash **dependencies_ptr, int calls)
 {
   int flag = 0;
   int i;
@@ -345,6 +328,13 @@ plugin_next_unrun_dependency (plugins_scheduler_t sched,
   if (dependencies_ptr == NULL)
     return NULL;
 
+  if (calls > 100)
+    {
+      log_write ("Possible dependency cycle detected %s",
+                 dependencies_ptr[0]->plugin->oid);
+      return NULL;
+    }
+
   for (i = 0; dependencies_ptr[i] != NULL; i++)
     {
       struct scheduler_plugin *plugin;
@@ -364,7 +354,7 @@ plugin_next_unrun_dependency (plugins_scheduler_t sched,
             if (deps_ptr == NULL)
               return plugin;
 
-            ret = plugin_next_unrun_dependency (sched, deps_ptr);
+            ret = plugin_next_unrun_dependency (sched, deps_ptr, calls++);
             if (ret == NULL)
               return plugin;
 
@@ -399,169 +389,183 @@ plugin_next_unrun_dependency (plugins_scheduler_t sched,
  */
 static void
 enable_plugin_and_dependencies (plugins_scheduler_t shed,
-                                struct arglist *plugin, char *name,
+                                struct scheduler_plugin *plugin,
                                 GHashTable *deps_table)
 {
   struct hash **deps_ptr;
-  int status;
   int i;
 
   if (plugin == NULL)
     return;
 
-  if (g_hash_table_lookup (deps_table, name))
+  if (g_hash_table_lookup (deps_table, plugin->oid))
     return;
   else
-    g_hash_table_insert (deps_table, g_strdup (name), name);
-
-  deps_ptr = hash_get_deps_ptr (shed->hash, name);
-
-  status = plug_get_launch (plugin);
-  if (status == LAUNCH_DISABLED)
-    plug_set_launch (plugin, LAUNCH_RUN);
+    g_hash_table_insert (deps_table, g_strdup (plugin->oid), plugin->oid);
 
+  plugin->enabled = TRUE;
+  deps_ptr = hash_get_deps_ptr (shed->hash, plugin->oid);
   if (deps_ptr == NULL)
     return;
   for (i = 0; deps_ptr[i] != NULL; i++)
     {
       struct scheduler_plugin *p;
       p = deps_ptr[i]->plugin;
-      if (p != NULL && p->arglist != NULL)
-        enable_plugin_and_dependencies (shed, p->arglist->value,
-                                        (char *)nvticache_get_filename ((const char *)p->arglist->name),
-                                        deps_table);
+      if (p)
+        enable_plugin_and_dependencies (shed, p, deps_table);
     }
 }
 
 /*---------------------------------------------------------------------------*/
 
+/*
+ * Enable plugins in scheduler, from a list.
+ *
+ * param[in]    sched       Plugins scheduler.
+ * param[in]    oid_list    List of plugins to enable.
+ * param[in]    autoload    Whether to autoload dependencies.
+ */
+static void
+plugins_scheduler_enable (plugins_scheduler_t sched, const char *oid_list,
+                          int autoload)
+{
+  int i;
+  char *oids, *oid;
+  GHashTable *oids_table;
+
+  oids_table = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, NULL);
+
+  /* Store list of plugins in hashtable. */
+  oids = g_strdup (oid_list);
+  oid = strtok (oids, ";");
+  while (oid)
+    {
+      g_hash_table_insert (oids_table, oid, oid);
+      oid = strtok (NULL, ";");
+    }
+
+  /* Enable plugins found in hashtable. */
+  for (i = ACT_FIRST; i <= ACT_LAST; i++)
+    {
+      struct list *element = sched->list[i];
+
+      while (element)
+        {
+          if (g_hash_table_lookup (oids_table, element->plugin->oid))
+            element->plugin->enabled = TRUE;
+
+          element = element->next;
+        }
+    }
+  g_hash_table_destroy (oids_table);
+  g_free (oids);
+
+  if (autoload != 0)
+    {
+      for (i = ACT_FIRST; i <= ACT_LAST; i++)
+        {
+          struct list *element = sched->list[i];
+
+          while (element)
+            {
+              /* deps_table is used to prevent circular dependencies. */
+              GHashTable *deps_table;
+
+              deps_table = g_hash_table_new_full
+                            (g_str_hash, g_str_equal, g_free, NULL);
+              if (element->plugin->enabled)
+                enable_plugin_and_dependencies (sched, element->plugin,
+                                                deps_table);
+
+              g_hash_table_destroy (deps_table);
+              element = element->next;
+            }
+        }
+    }
+}
+
+static void
+plugins_scheduler_fill (plugins_scheduler_t sched)
+{
+  int i;
+  GSList *list, *element;
+
+  list = element = nvticache_get_oids ();
+  while (element)
+    {
+      struct scheduler_plugin *scheduler_plugin;
+      struct list *dup;
+      int category;
+
+      category = nvticache_get_category (element->data);
+      scheduler_plugin = g_malloc0 (sizeof (struct scheduler_plugin));
+      scheduler_plugin->running_state = PLUGIN_STATUS_UNRUN;
+      scheduler_plugin->oid = g_strdup (element->data);
+      scheduler_plugin->enabled = FALSE;
+
+      assert (category <= ACT_LAST);
+      dup = g_malloc0 ( sizeof (struct list));
+      dup->plugin = scheduler_plugin;
+      dup->prev = NULL;
+      dup->next = sched->list[category];
+      if (sched->list[category] != NULL)
+        sched->list[category]->prev = dup;
+      sched->list[category] = dup;
+
+      hash_add (sched->hash, scheduler_plugin);
+      element = element->next;
+    }
+  g_slist_free_full (list, g_free);
+
+  for (i = 0; i < HASH_MAX; i++)
+    {
+      struct hash *l = &sched->hash[i];
+      while (l != NULL)
+        {
+          hash_fill_deps (sched->hash, l);
+          l = l->next;
+        }
+    }
+
+}
+
 plugins_scheduler_t
-plugins_scheduler_init (struct arglist *plugins, int autoload,
-                        int only_network)
+plugins_scheduler_init (const char *plugins_list, int autoload, int only_network)
 {
   plugins_scheduler_t ret;
-  struct arglist *arg;
   int i;
-  struct hash *l;
-
-  if (plugins == NULL)
-    return NULL;
 
   /* Fill our lists */
   ret = g_malloc0 (sizeof (*ret));
   ret->hash = hash_init ();
-  arg = plugins;
-  while (arg->next != NULL)
-    {
-      struct scheduler_plugin *scheduler_plugin;
-      struct list *dup;
-      char *oid = arg->name;
-      nvti_t *nvti;
-      int category;
+  plugins_scheduler_fill (ret);
 
-      if (!oid)
-        continue; // This would be a serious problem as it shouldn't be
-      nvti = nvticache_get_by_oid_full (oid);
-      category = nvti_category (nvti);
-      scheduler_plugin = g_malloc0 (sizeof (struct scheduler_plugin));
-      scheduler_plugin->arglist = arg;
-      scheduler_plugin->running_state = PLUGIN_STATUS_UNRUN;
-      scheduler_plugin->category = category;
-      scheduler_plugin->timeout = nvti_timeout (nvti);
-
-      if (nvti_required_ports (nvti) != NULL)
-        scheduler_plugin->required_ports = g_strsplit (nvti_required_ports (nvti), ", ", 0);
-      else
-        scheduler_plugin->required_ports = NULL;
-
-      if (nvti_required_udp_ports (nvti) != NULL)
-        scheduler_plugin->required_udp_ports = g_strsplit (nvti_required_udp_ports (nvti), ", ", 0);
-      else
-        scheduler_plugin->required_udp_ports = NULL;
-
-      if (nvti_required_keys (nvti) != NULL)
-        scheduler_plugin->required_keys = g_strsplit (nvti_required_keys (nvti), ", ", 0);
-      else
-        scheduler_plugin->required_keys = NULL;
-
-      if (nvti_mandatory_keys (nvti) != NULL)
-        scheduler_plugin->mandatory_keys = g_strsplit (nvti_mandatory_keys (nvti), ", ", 0);
-      else
-        scheduler_plugin->mandatory_keys = NULL;
-
-      if (nvti_excluded_keys (nvti) != NULL)
-        scheduler_plugin->excluded_keys = g_strsplit (nvti_excluded_keys (nvti), ", ", 0);
-      else
-        scheduler_plugin->excluded_keys = NULL;
-
-      nvti_free (nvti);
-
-      if (category > ACT_LAST)
-        category = ACT_LAST;
-      dup = g_malloc0 ( sizeof (struct list));
-      dup->plugin = scheduler_plugin;
-      dup->prev = NULL;
-      dup->next = ret->list[category];
-      if (ret->list[category] != NULL)
-        ret->list[category]->prev = dup;
-      ret->list[category] = dup;
-
-      hash_add (ret->hash, arg->name, scheduler_plugin);
-      arg = arg->next;
-    }
-
-  for (i = 0; i < HASH_MAX; i++)
-    {
-      l = &ret->hash[i];
-      while (l != NULL)
-        {
-          hash_fill_deps (ret->hash, l);
-          l = l->next;
-        }
-    }
-
-  if (autoload != 0)
-    {
-      arg = plugins;
-      while (arg->next != NULL)
-        {
-          /* deps_table is used to prevent circular dependencies. */
-          GHashTable *deps_table;
-
-          deps_table = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
-                                              NULL);
-          if (plug_get_launch (arg->value) != LAUNCH_DISABLED)
-            enable_plugin_and_dependencies
-             (ret, arg->value, (char *) nvticache_get_filename (arg->name),
-              deps_table);
-          arg = arg->next;
-          g_hash_table_destroy (deps_table);
-        }
-    }
+  plugins_scheduler_enable (ret, plugins_list, autoload);
 
   /* Now, remove the plugins that won't be launched */
   for (i = ACT_FIRST; i <= ACT_LAST; i++)
     {
-      struct list *l = ret->list[i];
-      while (l != NULL)
+      struct list *plist = ret->list[i];
+
+      while (plist != NULL)
         {
-          if (plug_get_launch (l->plugin->arglist->value) == LAUNCH_DISABLED)
+          if (!plist->plugin->enabled)
             {
-              struct list *old = l->next;
+              struct list *old = plist->next;
 
-              if (l->prev != NULL)
-                l->prev->next = l->next;
+              if (plist->prev != NULL)
+                plist->prev->next = plist->next;
               else
-                ret->list[i] = l->next;
+                ret->list[i] = plist->next;
 
-              if (l->next != NULL)
-                l->next->prev = l->prev;
+              if (plist->next != NULL)
+                plist->next->prev = plist->prev;
 
-              g_free (l);
-              l = old;
+              g_free (plist);
+              plist = old;
               continue;
             }
-          l = l->next;
+          plist = plist->next;
         }
     }
 
@@ -575,7 +579,25 @@ plugins_scheduler_init (struct arglist *plugins, int autoload,
   return ret;
 }
 
+int
+plugins_scheduler_count_active (plugins_scheduler_t sched)
+{
+  int ret = 0, i;
+  assert (sched);
 
+  for (i = ACT_FIRST; i <= ACT_LAST; i++)
+    {
+      struct list *element = sched->list[i];
+
+      while (element)
+        {
+          if (element->plugin->enabled)
+            ret++;
+          element = element->next;
+        }
+    }
+  return ret;
+}
 
 struct scheduler_plugin *
 plugins_scheduler_next (plugins_scheduler_t h)
@@ -613,7 +635,7 @@ plugins_scheduler_next (plugins_scheduler_t h)
                 if (deps_ptr != NULL)
                   {
                     struct scheduler_plugin *p =
-                      plugin_next_unrun_dependency (h, deps_ptr);
+                      plugin_next_unrun_dependency (h, deps_ptr, 0);
 
                     switch (GPOINTER_TO_SIZE (p))
                       {
@@ -625,9 +647,13 @@ plugins_scheduler_next (plugins_scheduler_t h)
                         break;
                       case GPOINTER_TO_SIZE (PLUG_RUNNING):
                         {
-                          /* One of the dependency is still running  -  we write down its category */
-                          if (l->plugin->category < running_category)
-                            running_category = l->plugin->category;
+                          /* One of the dependency is still running
+                           * we write down its category */
+
+                          int category = nvticache_get_category
+                                          (l->plugin->oid);
+                          if (category < running_category)
+                            running_category = category;
                           flag++;
                         }
                         break;
@@ -650,8 +676,9 @@ plugins_scheduler_next (plugins_scheduler_t h)
               break;
             case PLUGIN_STATUS_RUNNING:
               {
-                if (l->plugin->category < running_category)
-                  running_category = l->plugin->category;
+                int category = nvticache_get_category (l->plugin->oid);
+                if (category < running_category)
+                  running_category = category;
                 flag++;
               }
               break;
@@ -702,7 +729,6 @@ plugins_scheduler_next (plugins_scheduler_t h)
   return flag != 0 ? PLUG_RUNNING : NULL;
 }
 
-
 void
 list_destroy (struct list *list)
 {
@@ -714,13 +740,12 @@ list_destroy (struct list *list)
     }
 }
 
-
 void
 plugins_scheduler_free (plugins_scheduler_t sched)
 {
   int i;
   hash_destroy (sched->hash);
-  for (i = ACT_FIRST; i < ACT_LAST; i++)
+  for (i = ACT_FIRST; i <= ACT_LAST; i++)
     list_destroy (sched->list[i]);
   g_free (sched);
 }
