@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: pluginscheduler.h 20185 2014-08-27 08:40:02Z jan $
+* $Id: pluginscheduler.h 25469 2016-06-08 19:50:29Z hdoreau $
 * Description: header for pluginscheduler.c
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -31,31 +31,18 @@
 #ifndef PLUGINSCHEDULER_H
 #define PLUGINSCHEDULER_H
 
-#include <glib.h> /* for gchar */
+#include <glib.h>
 
 struct hash;
 struct plugins_scheduler;
 
-/**
- * @brief States of scheduler_plugin.
- *
- * @todo Consider creating an enumeration.
- */
-#define LAUNCH_DISABLED 0
-#define LAUNCH_RUN      1
 
 struct scheduler_plugin
 {
-  int running_state;
-  int category;
-  int timeout;
-  gchar **required_ports;     /* Last element is NULL (created by g_strsplit) */
-  gchar **required_udp_ports; /* Last element is NULL (created by g_strsplit) */
-  gchar **required_keys;      /* Last element is NULL (created by g_strsplit) */
-  gchar **mandatory_keys;     /* Last element is NULL (created by g_strsplit) */
-  gchar **excluded_keys;      /* Last element is NULL (created by g_strsplit) */
-  struct arglist *arglist;
   struct hash *parent_hash;
+  char *oid;
+  int running_state;
+  gboolean enabled;
 };
 
 
@@ -69,9 +56,16 @@ typedef struct plugins_scheduler *plugins_scheduler_t;
 #define PLUGIN_STATUS_DONE_AND_CLEANED 	4
 
 
-plugins_scheduler_t plugins_scheduler_init (struct arglist *, int, int);
-struct scheduler_plugin *plugins_scheduler_next (plugins_scheduler_t);
+plugins_scheduler_t
+plugins_scheduler_init (const char *, int, int);
 
-void plugins_scheduler_free (plugins_scheduler_t);
+struct scheduler_plugin *
+plugins_scheduler_next (plugins_scheduler_t);
+
+int
+plugins_scheduler_count_active (plugins_scheduler_t);
+
+void
+plugins_scheduler_free (plugins_scheduler_t);
 
 #endif
