@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: plugs_req.h 20751 2014-11-02 18:58:52Z jan $
+* $Id: plugs_req.h 24032 2016-01-04 21:56:25Z kroosec $
 * Description: plugs_req.c header.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -31,7 +31,7 @@
 #ifndef PLUGINS_REQUIREMENTS_H__
 #define PLUGINS_REQUIREMENTS_H__
 
-#include <openvas/misc/kb.h>         /* for struct kb_item */
+#include <openvas/base/kb.h>         /* for struct kb_item */
 #include <openvas/misc/arglists.h>   /* for struct arglist */
 
 char *requirements_plugin (kb_t, struct scheduler_plugin *);
