@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: processes.c 22723 2015-07-08 12:54:33Z kroosec $
+* $Id: processes.c 24698 2016-03-17 13:21:11Z timopollmeier $
 * Description: Creates new threads.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -83,7 +83,7 @@ create_process (process_func_t function, void *argument)
   if (pid == 0)
     {
       init_child_signal_handlers ();
-      srand48 (getpid () + getppid () + (long) time (NULL));    /* RATS: ignore */
+      srand48 (getpid () + getppid () + (long) time (NULL));
       (*function) (argument);
       exit (0);
     }
