@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: sighand.h 7177 2010-03-30 14:54:21Z mattm $
+* $Id: sighand.h 21730 2015-02-27 14:49:27Z kroosec $
 * Description: headerfile for sighand.c.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -31,19 +31,10 @@
 #ifndef _OPENVAS_SIGHAND_H
 #define _OPENVAS_SIGHAND_H
 
-extern void (*openvas_signal (int signum, void (*handler) (int))) (int);
-extern void sighand_pipe ();
-extern void sighand_chld ();
-extern void sighand_alarm ();
-extern void sighand_alarm_plugin ();
-extern void sighand_term ();
-extern void sighand_int ();
-extern void sighand_kill ();
-extern void sighand_segv ();
-extern void sighand_sigusr1 ();
-extern void sighand_io ();
-extern void sighandler (int sign);
+void (*openvas_signal (int signum, void (*handler) (int))) (int);
+void sighand_chld ();
+void sighand_segv ();
 
-extern void let_em_die (int pid);
-extern void make_em_die (int sig);
+void let_em_die (int pid);
+void make_em_die (int sig);
 #endif
