@@ -1,5 +1,5 @@
 /* OpenVAS
-* $Id: utils.c 22004 2015-03-31 15:06:57Z kroosec $
+* $Id: utils.c 26457 2016-10-23 18:09:06Z jan $
 * Description: A bunch of miscellaneous functions, mostly file conversions.
 *
 * Authors: - Renaud Deraison <deraison@nessus.org> (Original pre-fork develoment)
@@ -33,7 +33,6 @@
 #include <sys/stat.h>  /* for stat() */
 
 #include <openvas/misc/network.h>    /* for stream_zero */
-#include <openvas/misc/plugutils.h>  /* for plug_get_launch */
 #include <openvas/misc/prefs.h>      /* for prefs_get() */
 
 #include "log.h"
@@ -174,27 +173,6 @@ get_max_checks_number (void)
 }
 
 
-/**
- * @brief Returns the number of plugins that will be launched.
- */
-int
-get_active_plugins_number (struct arglist *plugins)
-{
-  int num = 0;
-
-  if (plugins != NULL)
-    while (plugins->next != NULL)
-      {
-        if (plug_get_launch (plugins->value) != LAUNCH_DISABLED)
-          num++;
-        plugins = plugins->next;
-      }
-
-  return num;
-}
-
-/*--------------------------------------------------------------------*/
-
 /**
  * Determines if a process is alive - as reliably as we can
  */
@@ -251,8 +229,7 @@ is_scanner_only_pref (const char *pref)
       || !strcmp (pref, "negot_timeout")
       || !strcmp (pref, "force_pubkey_auth")
       || !strcmp (pref, "log_while_attack")
-      || !strcmp (pref, "ca_file") || !strcmp (pref, "key_file")
-      || !strcmp (pref, "cert_file") || !strcmp (pref, "be_nice")
+      || !strcmp (pref, "be_nice")
       || !strcmp (pref, "log_plugins_name_at_load")
       || !strcmp (pref, "nasl_no_signature_check")
       /* Preferences starting with sys_ are scanner-side only. */
@@ -267,7 +244,7 @@ is_scanner_only_pref (const char *pref)
 static void
 auth_send (int soc, char *data)
 {
-  int sent = 0;
+  unsigned int sent = 0;
   gsize length;
 
   if (soc < 0)
